import React, { forwardRef, useEffect, useImperativeHandle, useRef } from 'react';
import { loadMicroApp } from 'qiankun';
export const MicroApp = forwardRef((props, microAppRef) => {
    const { current } = useRef({
        containerRef: null,
        microApp: null,
    });
    const loadApp = () => {
        if (current.containerRef) {
            current.microApp = loadMicroApp({
                ...props.microAppConfig,
                container: current.containerRef,
            }, {
                sandbox: {
                    experimentalStyleIsolation: true,
                },
            });
        }
    };
    useEffect(() => {
        loadApp();
        return () => {
            current.microApp?.unmount();
        };
    }, []);
    useEffect(() => {
        if (current.microApp && current.microApp.update) {
            current.microApp.update({});
        }
    }, []);
    useImperativeHandle(microAppRef, () => {
        return {
            appStore: current,
        };
    });
    return (React.createElement("div", { className: `micro-app-${props.microAppConfig.name}`, ref: (ref) => {
            current.containerRef = ref;
        } }));
});
//# sourceMappingURL=micro-app.js.map