import { MicroAppConfig } from './micro-fe-app-config';
export interface MicroAppWebpackConfigOptions {
    type: 'base-app' | 'micro-app';
    port: number;
    appConfig: MicroAppConfig;
    isBuildMode: boolean;
    isAnalyzeMode?: boolean;
}
/**
 * 基座或者微应用的 webpack 配置封装
 *
 * @author yuzhanglong
 * @date 2021-10-03 19:32:18
 * @param options 相关配置选项，可参考上面的类型定义
 * @see MicroAppWebpackConfigOptions
 */
export declare const getMicroAppWebpackConfig: (options: MicroAppWebpackConfigOptions) => {
    mode: string;
    entry: string;
    devtool: string | boolean;
    cache: boolean | {
        type: string;
    };
    optimization: {
        runtimeChunk: string;
        minimize: boolean;
        minimizer: any[];
        splitChunks: {
            chunks: string;
            cacheGroups: {
                thirdVendors: {
                    name: string;
                    test: RegExp;
                    priority: number;
                    enforce: boolean;
                };
                reactVendors: {
                    name: string;
                    test: RegExp;
                    priority: number;
                    enforce: boolean;
                };
                uiComponents: {
                    name: string;
                    test: RegExp;
                    priority: number;
                    enforce: boolean;
                };
                uiIcons: {
                    name: string;
                    test: RegExp;
                    priority: number;
                    enforce: boolean;
                };
                uiOthers: {
                    name: string;
                    test: RegExp;
                    priority: number;
                    enforce: boolean;
                };
            };
        };
    };
    devServer: {
        client: {
            webSocketURL: string;
        };
        static: {
            directory: string;
            watch: {
                ignored: (f: string) => boolean;
            };
        };
        allowedHosts: string;
        hot: boolean;
        port: number;
        historyApiFallback: boolean;
        headers: {
            'Access-Control-Allow-Origin': string;
        };
    };
    output: {
        library: string;
        filename: string;
        chunkFilename: string;
        assetModuleFilename: string;
        publicPath: string;
        libraryTarget: string;
    };
    plugins: any[];
    resolve: {
        extensions: string[];
        alias: {
            '~src': string;
        };
    };
    module: {
        rules: {
            oneOf: ({
                test: RegExp[];
                include: string[];
                use: {
                    loader: string;
                    options: {
                        presets: (string | {
                            runtime: string;
                        })[][];
                        plugins: (string | {
                            legacy: boolean;
                        })[][];
                    };
                };
                exclude?: undefined;
                type?: undefined;
            } | {
                test: RegExp[];
                use: (string | {
                    loader: string;
                    options: {
                        postcssOptions: {
                            plugins: any[];
                        };
                    };
                } | {
                    loader: string;
                    options: {
                        lessOptions: {
                            javascriptEnabled: boolean;
                        };
                    };
                })[];
                include?: undefined;
                exclude?: undefined;
                type?: undefined;
            } | {
                test: RegExp[];
                use: (string | {
                    loader: string;
                    options: {
                        postcssOptions: {
                            plugins: any[];
                        };
                    };
                } | {
                    loader: string;
                })[];
                include?: undefined;
                exclude?: undefined;
                type?: undefined;
            } | {
                exclude: RegExp[];
                type: string;
                test?: undefined;
                include?: undefined;
                use?: undefined;
            })[];
        }[];
    };
};
