import webpack from 'webpack';
declare type SharedLibraryExpose = string | {
    name: string;
    path: string;
    type?: 'package' | 'module';
};
declare type SharedLibrary = string | {
    name: string;
    type?: 'package' | 'module';
};
export interface MicroAppConfig {
    name: string;
    url: string;
    remotes: {
        name: string;
        url: string;
        sharedLibraries?: SharedLibrary[];
    }[];
    exposes: SharedLibraryExpose[];
    webpackConfig?: Partial<webpack.Configuration> | ((config: webpack.Configuration) => void);
}
/**
 * 基于 micro app config 生成目录
 *
 * @author yuzhanglong
 * @date 2021-09-28 00:40:39
 * @return object 一个对象, key 表示远程应用的名称，value 表示其入口 url
 */
export declare const getModuleFederationRemotes: (microAppConfig: MicroAppConfig) => Record<string, string>;
/**
 * 获取 share library 的导入替换回调函数
 *
 * 基座暴露一些公共库，我们称为 share library，我们通过 NormalModuleReplacementPlugin 将所有的公共依赖导向相应的 app
 *
 * @author yuzhanglong
 * @date 2021-09-28 00:47:33
 */
export declare const getNormalModuleReplacementPluginCallBack: (microAppConfig: MicroAppConfig) => (v: {
    request: string;
}) => void;
/**
 * 初始化全局 manager 方便调用
 *
 * @author yuzhanglong
 * @date 2021-09-28 00:53:32
 */
export declare const getMicroAppConfigManager: (microAppConfig: MicroAppConfig) => {
    getNormalModuleReplacementPluginCallBack: () => (v: {
        request: string;
    }) => void;
    getModuleFederationRemotes: () => Record<string, string>;
    config: MicroAppConfig;
};
export {};
