import webpack from 'webpack';
import WebpackDevServer from 'webpack-dev-server';
export const webpackPromisify = (config) => {
    return new Promise((resolve, reject) => {
        webpack(config, (err, state) => {
            if (err) {
                console.error(err.stack || err);
                if (err.details) {
                    console.error(err.details);
                }
                reject();
            }
            const info = state.toJson();
            if (state.hasErrors()) {
                console.error(info.errors);
                reject();
            }
            if (state.hasWarnings()) {
                console.warn(info.warnings);
                reject();
            }
            resolve(true);
        });
    });
};
export const webpackBuild = async (config) => {
    await webpackPromisify(config);
};
export const webpackServe = async (config) => {
    const compiler = webpack(config);
    const devServerOptions = {
        ...config.devServer,
    };
    // @ts-ignore
    const server = new WebpackDevServer(devServerOptions, compiler);
    server.startCallback(() => {
        console.log('webpack dev server is running...');
    });
};
//# sourceMappingURL=webpack-command.js.map