"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.emitMfExposeDeclaration = void 0;
const path = __importStar(require("path"));
const fs = __importStar(require("fs-extra"));
const bundle_ts_declaration_1 = require("./bundle-ts-declaration");
const bundle_module_declare_1 = require("./bundle-module-declare");
/**
 * 供公共组件的提供者使用，用来将相应的类型定义写入某个文件目录下
 * 我们可以在 webpack 写入文件之后 hook 到相应的生命周期中追加内容
 *
 * @author yuzhanglong
 * @date 2021-10-02 14:51:10
 * @param appConfig app 配置
 * @param baseUrl 写入的根路径
 */
const emitMfExposeDeclaration = async (appConfig, baseUrl) => {
    // 增加临时缓存文件，用来打包每个小 bundle
    await fs.ensureDir(path.resolve(baseUrl, '.cache'));
    const entries = [];
    for (const expose of appConfig.exposes) {
        // 只处理 module 类型
        if (typeof expose === 'object' && expose.type !== 'package') {
            entries.push({
                name: expose.name,
                entryPath: expose.path,
                outputPath: path.resolve(baseUrl, '.cache', `${expose.name}.d.ts`),
            });
        }
    }
    // 并行打包
    await (0, bundle_ts_declaration_1.bundleTsDeclaration)(entries.slice());
    // 合并上面的所有小 bundle
    const content = (0, bundle_module_declare_1.bundleModuleDeclare)(entries.map(res => {
        return {
            path: res.outputPath,
            moduleName: `${appConfig.name}/${res.name}`,
        };
    }));
    await fs.writeFile(path.resolve(baseUrl, 'exposes.d.ts'), content);
    await fs.remove(path.resolve(baseUrl, '.cache'));
};
exports.emitMfExposeDeclaration = emitMfExposeDeclaration;
//# sourceMappingURL=emit-mf-expose-declaration.js.map