"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EmitMfExposeWebpackPlugin = exports.debounce = void 0;
const path = __importStar(require("path"));
const emit_mf_expose_declaration_1 = require("./emit-mf-expose-declaration");
/**
 * 防抖函数
 *
 * @author yuzhanglong
 * @date 2021-12-01 00:51:19
 */
const debounce = (fn, timeout = 0) => {
    if (typeof fn !== 'function') {
        throw new Error('fn should be a function');
    }
    let timer = null;
    return function (...args) {
        if (timer) {
            clearTimeout(timer);
        }
        timer = setTimeout(() => {
            fn.call(this, ...args);
        }, timeout);
    };
};
exports.debounce = debounce;
/**
 * 向 build 打包产物注入类型定义的 webpack-plugin
 *
 * @author yuzhanglong
 * @date 2021-10-03 19:31:02
 */
class EmitMfExposeWebpackPlugin {
    config;
    constructor(config) {
        this.config = config;
    }
    apply(compiler) {
        const { appConfig, outputBasePath } = this.config;
        const handler = (0, exports.debounce)(async (compilation) => {
            try {
                // 用 try catch 包裹一下防止 webpack-dev-server 热更新过程中偶发的强制 exit 现象
                if (appConfig) {
                    // 拿到本项目的 outputPath
                    const { outputPath } = compilation.compiler;
                    // 生成相关目录
                    const target = path.resolve(outputBasePath ?? outputPath, 'mf-expose-types');
                    console.log('[mf-lite] compiling shared remote module declarations...');
                    // 基于用户的配置 appConfig 生成类型定义
                    await (0, emit_mf_expose_declaration_1.emitMfExposeDeclaration)(appConfig, target);
                }
            }
            catch (e) {
                console.log(e);
            }
        }, 1500);
        // afterEmit 生命周期的时机：输出 asset 到 output 目录之后
        // 实践证明，它不会阻塞 webpack dev-server 的流程，不会影响开发体验。
        compiler.hooks.afterEmit.tap('EmitMfExposeWebpackPlugin', handler);
        // TODO: 使用文件 hash 进行缓存，避免相同的内容重复打包，可以参考下面的的注释 DEMO
        // compiler.hooks.thisCompilation.tap('EmitMfExposeWebpackPlugin', (compilation) => {
        //   compilation.hooks.processAssets.tapAsync({
        //     name: 'EmitMfExposeWebpackPlugin',
        //     stage: webpack.Compilation.PROCESS_ASSETS_STAGE_OPTIMIZE_INLINE,
        //   }, (compilationAssets, callback) => {
        //     // 首先收集所有的文件，进行 content hash 值比对
        //     // @ts-ignore
        //     const data = [...compilation.modules].filter(res => res.request && res.request.includes('shared-utils.ts'))[0];
        //     console.log(data.buildInfo);
        //     return callback();
        //   });
        // });
    }
}
exports.EmitMfExposeWebpackPlugin = EmitMfExposeWebpackPlugin;
//# sourceMappingURL=emit-mf-expose-webpack-plugin.js.map