"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.webpackServe = exports.webpackBuild = exports.webpackPromisify = void 0;
const webpack_1 = __importDefault(require("webpack"));
const webpack_dev_server_1 = __importDefault(require("webpack-dev-server"));
const webpackPromisify = (config) => {
    return new Promise((resolve, reject) => {
        (0, webpack_1.default)(config, (err, state) => {
            if (err) {
                console.error(err.stack || err);
                if (err.details) {
                    console.error(err.details);
                }
                reject();
            }
            const info = state.toJson();
            if (state.hasErrors()) {
                console.error(info.errors);
                reject();
            }
            if (state.hasWarnings()) {
                console.warn(info.warnings);
                reject();
            }
            resolve(true);
        });
    });
};
exports.webpackPromisify = webpackPromisify;
const webpackBuild = async (config) => {
    await (0, exports.webpackPromisify)(config);
};
exports.webpackBuild = webpackBuild;
const webpackServe = async (config) => {
    const compiler = (0, webpack_1.default)(config);
    const devServerOptions = {
        ...config.devServer,
    };
    // @ts-ignore
    const server = new webpack_dev_server_1.default(devServerOptions, compiler);
    server.startCallback(() => {
        console.log('webpack dev server is running...');
    });
};
exports.webpackServe = webpackServe;
//# sourceMappingURL=webpack-command.js.map